package ru.amse.cat.evlarchick.ui;

import ru.amse.cat.evlarchick.save.ITextStructureSaver;
import ru.amse.cat.evlarchick.save.TextStructureSaver;
import ru.amse.cat.evlarchick.save.SavingException;
import ru.amse.cat.evlarchick.textstructure.ITextStructure;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import java.awt.event.*;
import java.io.File;
import java.io.IOException;

public class MainFrame extends JFrame {
    private JFileChooser myFileChoser = new JFileChooser();
    private ITextStructureSaver mySaver = new TextStructureSaver();
    private ITextStructure myTextStructure;
    private File myOpenedFile;

    public MainFrame() {
        addMenuBar();
        addWindowListener(new WindowAdapter(){
            public void windowClosing(WindowEvent e) {
                if (myTextStructure == null) {
                    return;
                }
                int n = JOptionPane.showConfirmDialog(MainFrame.this, "Do you wanna save the changes?", "Confirm Exit",
                        JOptionPane.YES_NO_OPTION, JOptionPane.CLOSED_OPTION);
                if (n == 0) {
                    processSaving();
                }
            }
        });
        setContentPane(new MainTextPanel());
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        pack();
        setVisible(true);
    }

    private void addMenuBar() {
        JMenuBar menu = new JMenuBar();

        JMenu file = new JMenu("File");

        AbstractAction open = new AbstractAction("Open...") {
            {
                putValue(Action.MNEMONIC_KEY, Integer.valueOf('o'));
                putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_O, KeyEvent.CTRL_MASK));
            }

            public void actionPerformed(ActionEvent ae) {
                myFileChoser.setFileSelectionMode(JFileChooser.FILES_ONLY);
                myFileChoser.setFileFilter(new FileFilter() {
                    public boolean accept(File f) {
                        String filepath = f.getName();
                        return filepath.endsWith(".txt") || filepath.endsWith(".cat") || f.isDirectory();
                    }
                    public String getDescription() {
                        return "";
                    }
                });
                int returnValue = myFileChoser.showOpenDialog(MainFrame.this);

                if (returnValue == JFileChooser.APPROVE_OPTION) {
                    myOpenedFile = myFileChoser.getSelectedFile();
                    try {
                        myTextStructure = mySaver.load(myOpenedFile);
                        if (myTextStructure != null) {
                            MainTextPanel mtp = new MainTextPanel();
                            UIManager.getInstance().setTextStructure(myTextStructure);
                            mtp.add(UIManager.getInstance().createInitialTextPanel());
                            mtp.add(UIManager.getInstance().createTranslationPanel());
                            setContentPane(mtp);
                            pack();
                        }
                    } catch (SavingException e) {
                        JOptionPane.showMessageDialog(MainFrame.this, "Bad file",
                                "Error", JOptionPane.ERROR_MESSAGE);
                    }
                }
            }
        };

        AbstractAction saveAs = new AbstractAction("Save as...") {
            {
                putValue(Action.MNEMONIC_KEY, Integer.valueOf('s'));
                putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke("control shift S"));
            }


            public void actionPerformed(ActionEvent ae) {
                processSaveToNewFile();
            }
        };

        AbstractAction save = new AbstractAction("Save...") {
            {
                putValue(Action.MNEMONIC_KEY, Integer.valueOf('s'));
                putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke("control S"));
            }

            public void actionPerformed(ActionEvent e) {
                processSaving();
            }
        };

        file.add(open);
        file.add(save);
        file.add(saveAs);
        menu.add(file);

        JMenu edit = new JMenu("Edit");

        AbstractAction find = new AbstractAction("Find...") {
            {
                putValue(Action.MNEMONIC_KEY, Integer.valueOf('f'));
                putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_F, KeyEvent.CTRL_MASK));
            }

            public void actionPerformed(ActionEvent ae) {
                new FindDialog(MainFrame.this, FindDialog.FIND).setVisible(true);
            }
        };

        AbstractAction replace = new AbstractAction("Replace...") {
            {
                putValue(Action.MNEMONIC_KEY, Integer.valueOf('r'));
                putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_R, KeyEvent.CTRL_MASK));
            }

            public void actionPerformed(ActionEvent ae) {
                new FindDialog(MainFrame.this, FindDialog.REPLACE).setVisible(true);
            }
        };

        edit.add(find);
        edit.add(replace);
        menu.add(edit);

        setJMenuBar(menu);
    }

    private void processSaving() {
        boolean saveToOpenedFile = mySaver.wasCATFileLoaded();

        if (saveToOpenedFile) {
            try {
                mySaver.save(myTextStructure, myOpenedFile);
                savedSuccesfully();
            } catch (SavingException e1) {
                JOptionPane.showMessageDialog(this, "Cannot save",
                        "Error", JOptionPane.ERROR_MESSAGE);
            }
        } else {
            processSaveToNewFile();
        }
    }

    private void savedSuccesfully() {
        JOptionPane.showMessageDialog(this, "Saved Succesfully");
    }

    private void processSaveToNewFile() {
        if (myOpenedFile == null) {
            return;
        }

        myFileChoser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        myFileChoser.setFileFilter(new FileFilter() {
            public boolean accept(File f) {
                String filepath = f.getName();
                return filepath.endsWith(".txt") || filepath.endsWith(".cat") || f.isDirectory();
            }
            public String getDescription() {
                return "";
            }
        });
        int returnValue = myFileChoser.showSaveDialog(MainFrame.this);

        if (returnValue == JFileChooser.APPROVE_OPTION) {
            File f = myFileChoser.getSelectedFile();
            try {
                if (!f.exists()) {
                    f.createNewFile();
                }

                mySaver.save(myTextStructure, f);
                myOpenedFile = f;
                savedSuccesfully();
            } catch (IOException e) {
                JOptionPane.showMessageDialog(MainFrame.this, "Bad file",
                        "Error", JOptionPane.ERROR_MESSAGE);
            } catch (SavingException e) {
                JOptionPane.showMessageDialog(MainFrame.this, "Bad file",
                        "Error", JOptionPane.ERROR_MESSAGE);
            }
        }
    }
}
