package ru.amse.cat.evlarchick.ui;

import ru.amse.cat.evlarchick.textstructure.*;

import javax.swing.*;
import java.util.*;
import java.io.StringReader;

public class UIManager {
    private static final Object[] MY_OPTIONS = new Object[] {"With Translation", "Dead Zone", "Cancel"};

    private static UIManager ourInstance;

    private HotTextPane myInitialTextPanel;
    private HotTextPane myTranslationPanel;
    private ITextInfo myTextInfo;
    private ITextInfo myTranslationInfo;
    private ITextStructure myTextStructure;
    private Finder myFinder;

    private UIManager() {
        myFinder = new Finder();
    }

    public static UIManager getInstance() {
        if (ourInstance == null) {
            ourInstance = new UIManager();
        }
        return ourInstance;
    }

    public void setTextStructure(ITextStructure ts) {
        myTextStructure = ts;
        myFinder.setTextStructure(ts);
    }

    public HotTextPane createTranslationPanel() {
        myTranslationInfo = new TextInfo(myTextStructure.getTranslationReader());
        myFinder.setTanslationInfo(myTranslationInfo);

        refreshTranslationInfo();
        myTranslationPanel = new HotTextPane(myTranslationInfo);
        myTranslationPanel.addSegmentListener(new TranslationListener());
        return myTranslationPanel;
    }

    public HotTextPane createInitialTextPanel() {
        myTextInfo = new TextInfo(new StringReader(myTextStructure.getOriginalText()),
                myTextStructure.getSegments(), myTextStructure.getDeadZones());
        myFinder.setInitialTextInfo(myTextInfo);

        myInitialTextPanel = new HotTextPane(myTextInfo);
        myInitialTextPanel.addSegmentListener(new OriginalTextListener());
        return myInitialTextPanel;
    }

    private void refreshTranslationInfo() {
        myTranslationInfo.clear();

        Iterator<ISegmentWithTranslation> segIterator = myTextStructure.getSegments().iterator();
        Iterator<ISegment> dzIterator = myTextStructure.getDeadZones().iterator();

        ISegment deadZone = null;
        ISegmentWithTranslation segment = null;

        int textCount = 0;
        int translationCount = 0;
        int mode = 0;

        do {
            ISegment curent;
            if (mode >= 0) {
                deadZone = (dzIterator.hasNext()) ? dzIterator.next() : null;
            }
            if (mode <= 0) {
                segment = (segIterator.hasNext()) ? segIterator.next() : null;
            }
            if (deadZone == null && segment == null) {
                break;
            } else if (deadZone == null) {
                mode = -1;
                curent = segment;
            } else if (segment == null) {
                mode = 1;
                curent = deadZone;
            } else if (deadZone.getStart() < segment.getStart()) {
                mode = 1;
                curent = deadZone;
            } else {
                mode = -1;
                curent = segment;
            }

            translationCount += curent.getStart() - textCount;
            textCount =  curent.getEnd();
            int length = (curent instanceof ISegmentWithTranslation) ?
                    ((ISegmentWithTranslation)curent).getTranslation().length() :
                    curent.getEnd() - curent.getStart();
            ISegment s = new SimpleSegment(translationCount, translationCount + length);
            if (mode == -1) {
                myTranslationInfo.addSegment(s);
            } else {
                myTranslationInfo.addDeadZone(s);
            }
            translationCount += length;
        } while (true);
    }

    public void processFind(String text, boolean isDZ) {
        myFinder.processFind(text, isDZ);
        myTranslationPanel.repaint();
        myInitialTextPanel.repaint();
    }

    public void processReplace(String text, String replaceWith) {
        myFinder.processReplace(text, replaceWith);
        myTranslationPanel.setLinesWereModified();
        myTranslationPanel.repaint();
        myInitialTextPanel.repaint();
    }

    private int getOriginalIndexFromTransltional(int beg) {
        List<ISegmentWithTranslation> segmentsFromTS = myTextStructure.getSegments();
        List<ISegment> trSegments = myTranslationInfo.getSegments();

        int result = beg;
        for (int i = 0; i < trSegments.size(); i++) {
            ISegment s = trSegments.get(i);
            if (s.getEnd() < beg) {
                ISegmentWithTranslation seg = segmentsFromTS.get(i);
                result -= seg.getTranslation().length() - seg.getText().length();
            } else {
                break;
            }
        }
        return result;
    }

    private abstract class TextListener implements ISegmentListener {

        public void segmentClicked(int index, boolean isDeadZone) {
            if (isDeadZone) {
                myInitialTextPanel.setMarked(myTextInfo.getDeadZones().get(index));
                myTranslationPanel.setMarked(myTranslationInfo.getDeadZones().get(index));
            } else {
                myInitialTextPanel.setMarked(myTextInfo.getSegments().get(index));
                myTranslationPanel.setMarked(myTranslationInfo.getSegments().get(index));
            }
        }

        public void segmentTranslationChanged(int i) {
            new SegmentTranslationDialog(myTextStructure.getSegments().get(i)).setVisible(true);
            refreshTranslationInfo();
            myTranslationPanel.setMarked(myTranslationInfo.getSegments().get(i));
            myTranslationPanel.setLinesWereModified();
            myTranslationPanel.repaint();
        }

        protected void createSegment(int[] indicesFromTS, HotTextPane root) {
            int n = JOptionPane.showOptionDialog(root, "What kind of segment do you want to create?",
                    "Segment type selection", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE,
                    null, MY_OPTIONS, MY_OPTIONS[2]);

            if (n == 0) {
                try {
                    ISegmentWithTranslation s = new SegmentWithTranslation(
                            myTextStructure.getOriginalText().substring(indicesFromTS[0], indicesFromTS[1]),
                            indicesFromTS[0], indicesFromTS[1], myTextStructure);
                    myTextStructure.addSegmentWithTranslation(s);
                    myTextInfo.addSegment(new SimpleSegment(indicesFromTS[0], indicesFromTS[1]));
                    new SegmentTranslationDialog(s).setVisible(true);
                    refreshTranslationInfo();
                } catch (SegmentIsTooBigException e) {
                    e.printStackTrace();
                }
            } else if (n == 1) {
                try {
                    ISegment s = new Segment(indicesFromTS[0], indicesFromTS[1], myTextStructure);
                    myTextStructure.addDeadZone(s);
                    myTextInfo.addDeadZone(new SimpleSegment(indicesFromTS[0], indicesFromTS[1]));
                    refreshTranslationInfo();
                } catch (SegmentIsTooBigException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    private class OriginalTextListener extends TextListener {

        public void segmentClicked(int index, boolean isDeadZone) {
            super.segmentClicked(index, isDeadZone);
            myTranslationPanel.repaint();
        }

        public void segmentCreated(int beg, int end) {
            int[] indicesFromTS = new int[2];

            indicesFromTS[0] = beg;
            indicesFromTS[1] = end;

            createSegment(indicesFromTS, myInitialTextPanel);

            myTranslationPanel.setLinesWereModified();
            myTranslationPanel.repaint();
        }
    }

    private class TranslationListener extends TextListener {

        public void segmentClicked(int index, boolean isDeadZone) {
            super.segmentClicked(index, isDeadZone);
            myInitialTextPanel.repaint();
        }

        public void segmentCreated(int beg, int end) {
            int[] indicesFromTS = new int[2];
            indicesFromTS[0] = getOriginalIndexFromTransltional(beg);
            indicesFromTS[1] = indicesFromTS[0] + end - beg;

            createSegment(indicesFromTS, myTranslationPanel);
            myTranslationPanel.setLinesWereModified();
            myInitialTextPanel.repaint();
        }
    }
}
